/* sockbind.c
 * 
 * Copyright (C) 2004 Simone Piccardi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/****************************************************************
 *
 * Routine sockbind
 * Return a binded socket given hostname, service, and socket type
 *
 * Author: Simone Piccardi
 * Dec. 2004
 *
 * $Id$ 
 *
 ****************************************************************/
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>

int sockbind(char *host, char *serv, int prot, int type) 
{
    struct addrinfo hint, *addr, *save;
    int res;
    int sock;
    /* initialize hint structure */
    memset(&hint, 0, sizeof(struct addrinfo)); 
    hint.ai_flags = AI_PASSIVE;          /* address for binding */
    hint.ai_family = PF_UNSPEC;          /* generic address (IPv4 or IPv6) */
    hint.ai_protocol = prot;             /* protocol */
    hint.ai_socktype = type;             /* socket type */
    res = getaddrinfo(host, serv, &hint, &addr);   /* calling getaddrinfo */
    if (res != 0) {                                /* on error exit */
	printf("sockbind cannot resolve host %s, service %s, ", host, serv);
	printf("protocol %d: %s\n", prot, gai_strerror(res));
	errno = 0;                                 /* clear errno */
	return -1;
    }
    /* loop on possible addresses */
    save = addr;
    while (addr != NULL) {
	/* get a socket */
	sock = socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
	if ( (sock < 0) && (addr->ai_next == NULL) ) {
	    printf("sockbind cannot create socket\n");
	    return sock;
	} else continue;
	/* connect the socket */
	res = bind(sock, addr->ai_addr, addr->ai_addrlen);
	if ( (res < 0) && (addr->ai_next == NULL) ) {
	    printf("sockbind cannot bind socket\n");
	    return res;
	}
	addr=addr->ai_next;
    }
    freeaddrinfo(save);                            /* done, release memory */
    return sock;
}
